package se.su.dsv.scipro.crosscutting;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import se.su.dsv.scipro.notifications.NotificationController;
import se.su.dsv.scipro.notifications.dataobject.NotificationSource;
import se.su.dsv.scipro.notifications.dataobject.ProjectEvent;
import se.su.dsv.scipro.project.ReviewerAssignedEvent;
import se.su.dsv.scipro.reviewing.FinalSeminarApprovalService;
import se.su.dsv.scipro.reviewing.ReviewerApproval;
import se.su.dsv.scipro.reviewing.RoughDraftApprovalService;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.function.Consumer;

@Singleton
class ReviewerAssignedNotifications {
    private final RoughDraftApprovalService roughDraftApprovalService;
    private final FinalSeminarApprovalService finalSeminarApprovalService;
    private final NotificationController notificationController;

    ReviewerAssignedNotifications(
            RoughDraftApprovalService roughDraftApprovalService,
            FinalSeminarApprovalService finalSeminarApprovalService,
            NotificationController notificationController) {
        this.roughDraftApprovalService = roughDraftApprovalService;
        this.finalSeminarApprovalService = finalSeminarApprovalService;
        this.notificationController = notificationController;
    }

    @Inject
    public ReviewerAssignedNotifications(
            RoughDraftApprovalService roughDraftApprovalService,
            FinalSeminarApprovalService finalSeminarApprovalService,
            NotificationController notificationController,
            EventBus eventBus) {
        this(roughDraftApprovalService, finalSeminarApprovalService, notificationController);
        eventBus.register(this);
    }

    @Subscribe
    public void reviewerAssigned(ReviewerAssignedEvent event) {
        roughDraftApprovalService.findBy(event.getProject())
                .filter(rda -> !rda.isDecided())
                .foreach(sendNotification(ProjectEvent.Event.ROUGH_DRAFT_APPROVAL_REQUESTED));

        finalSeminarApprovalService.findBy(event.getProject())
                .filter(fsa -> !fsa.isDecided())
                .foreach(sendNotification(ProjectEvent.Event.FINAL_SEMINAR_APPROVAL_REQUESTED));
    }

    private Consumer<ReviewerApproval> sendNotification(ProjectEvent.Event event) {
        return approval -> notificationController.notifyProject(approval.getProject(), event, new NotificationSource());
    }
}