package se.su.dsv.scipro.forum;

import com.querydsl.jpa.impl.JPAQuery;
import se.su.dsv.scipro.forum.dataobjects.*;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.GenericRepo;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

import static com.querydsl.core.types.dsl.Expressions.allOf;

class ForumPostRepositoryImpl extends GenericRepo<ForumPost, Long> implements ForumPostRepository {
    @Inject
    ForumPostRepositoryImpl(Provider<EntityManager> em) {
        super(em, ForumPost.class, QForumPost.forumPost);
    }

    @Override
    public List<ForumPost> findByThread(ForumThread forumThread) {
        return findAll(allOf(QForumPost.forumPost.forumThread.eq(forumThread), QForumPost.forumPost.deleted.isFalse()));
    }

    @Override
    public List<ForumPost> latestPost(Project project, int amount) {
        return createQuery()
                .where(QForumPost.forumPost.forumThread.in(
                        new JPAQuery<ForumThread>().from(QProjectThread.projectThread)
                                .where(QProjectThread.projectThread.project.eq(project))))
                .orderBy(QForumPost.forumPost.dateCreated.desc())
                .limit(amount)
                .fetch();
    }
}
