package se.su.dsv.scipro.grading;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.JsonTokenId;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.exc.InvalidFormatException;

import java.io.IOException;

public enum ReportGradeError {
    NOT_AUTHOR,
    NO_GRADING_ROLE,
    ALREADY_REPORTED,
    INVALID_EXAMINATION,
    NO_TEXT_MATCHING_DOCUMENT,
    INVALID_GRADE,
    NO_PUBLICATION_FILE
}

class ReportGradeErrorConverter extends JsonDeserializer<ReportGradeError> {
    @Override
    public ReportGradeError deserialize(final JsonParser p, final DeserializationContext ctxt) throws IOException, JsonProcessingException {
        if (p.currentTokenId() == JsonTokenId.ID_STRING) {
            final String text = p.getText();
            switch (text) {
                case "not.author":
                    return ReportGradeError.NOT_AUTHOR;
                case "no.grading.role":
                    return ReportGradeError.NO_GRADING_ROLE;
                case "already.reported":
                    return ReportGradeError.ALREADY_REPORTED;
                case "invalid.examination":
                    return ReportGradeError.INVALID_EXAMINATION;
                case "invalid.grade":
                    return ReportGradeError.INVALID_GRADE;
                case "no.text.matching.document":
                    return ReportGradeError.NO_TEXT_MATCHING_DOCUMENT;
                case "no.publication.file":
                    return ReportGradeError.NO_PUBLICATION_FILE;
                default:
                    throw new InvalidFormatException(p, "Unknown report grade error", text, Grade.Type.class);
            }
        }
        throw new InvalidFormatException(p, "Invalid report grade error", null, Grade.Type.class);
    }
}
