package se.su.dsv.scipro.milestones;

import se.su.dsv.scipro.milestones.dataobjects.MilestoneActivityTemplate;
import se.su.dsv.scipro.milestones.dataobjects.MilestonePhaseTemplate;
import se.su.dsv.scipro.milestones.dataobjects.QMilestoneActivityTemplate;
import se.su.dsv.scipro.system.GenericRepo;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static com.querydsl.core.types.dsl.Expressions.allOf;

public class MilestoneActivityTemplateRepositoryImpl extends GenericRepo<MilestoneActivityTemplate, Long> implements MilestoneActivityTemplateRepository {
    @Inject
    public MilestoneActivityTemplateRepositoryImpl(Provider<EntityManager> em) {
        super(em, MilestoneActivityTemplate.class, QMilestoneActivityTemplate.milestoneActivityTemplate);
    }

    @Override
    public List<MilestoneActivityTemplate> findActivities(MilestoneActivityTemplate.Type type, ProjectType projectType, Date projectStartDate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.type.eq(type),
                activity.projectTypes.any().eq(projectType),
                activity.deleted.isFalse(),
                activity.dateCreated.before(projectStartDate)
        )).fetch();
    }

    @Override
    public List<MilestoneActivityTemplate> findAllActivitiesByProjectType(MilestoneActivityTemplate.Type type, Collection<ProjectType> projectTypes) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        if (projectTypes.isEmpty()) {
            return Collections.emptyList();
        }
        return createQuery().where(allOf(
                activity.type.eq(type),
                activity.projectTypes.any().in(projectTypes),
                activity.deleted.isFalse()
        )).orderBy(activity.milestonePhaseTemplate.sortOrder.asc(), activity.sortOrder.asc()).fetch();
    }

    @Override
    public List<MilestoneActivityTemplate> findAllActivitiesInOrder(MilestoneActivityTemplate.Type type, ProjectType projectType, Date projectStartDate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.type.eq(type),
                activity.projectTypes.any().eq(projectType),
                activity.deleted.isFalse(),
                activity.dateCreated.before(projectStartDate)
        )).orderBy(activity.milestonePhaseTemplate.sortOrder.asc(), activity.sortOrder.asc()).fetch();
    }

    @Override
    public List<MilestoneActivityTemplate> findByMilestonePhase(MilestonePhaseTemplate milestonePhaseTemplate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(activity.milestonePhaseTemplate.eq(milestonePhaseTemplate)).orderBy(activity.sortOrder.asc()).fetch();
    }

    @Override
    public List<MilestoneActivityTemplate> findActiveByPhase(MilestonePhaseTemplate milestonePhaseTemplate, ProjectType projectType, Date projectStartDate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.milestonePhaseTemplate.eq(milestonePhaseTemplate),
                activity.deleted.isFalse(),
                activity.dateCreated.before(projectStartDate),
                activity.projectTypes.any().eq(projectType)
        )).orderBy(activity.sortOrder.asc()).fetch();
    }

    @Override
    public long countByMilestonePhase(MilestonePhaseTemplate milestonePhaseTemplate) {
        return createQuery().where(QMilestoneActivityTemplate.milestoneActivityTemplate.milestonePhaseTemplate.eq(milestonePhaseTemplate)).fetchCount();
    }

    @Override
    public long countActivities(MilestoneActivityTemplate.Type type, ProjectType projectType, Date projectStartDate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.type.eq(type),
                activity.projectTypes.any().eq(projectType),
                activity.deleted.isFalse(),
                activity.dateCreated.before(projectStartDate),
                activity.milestonePhaseTemplate.dateCreated.before(projectStartDate)
        )).fetchCount();
    }

    @Override
    public long countActivities(ProjectType projectType, Date projectStartDate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.projectTypes.any().eq(projectType),
                activity.deleted.isFalse(),
                activity.dateCreated.before(projectStartDate),
                activity.milestonePhaseTemplate.dateCreated.before(projectStartDate)
        )).fetchCount();
    }

    @Override
    public MilestoneActivityTemplate findBySortOrderAndMilestonePhaseTemplate(int sortOrder, MilestonePhaseTemplate milestonePhaseTemplate) {
        QMilestoneActivityTemplate activity = QMilestoneActivityTemplate.milestoneActivityTemplate;
        return createQuery().where(allOf(
                activity.sortOrder.eq(sortOrder),
                activity.milestonePhaseTemplate.eq(milestonePhaseTemplate)
        )).fetchOne();
    }
}
