package se.su.dsv.scipro.plagiarism;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.plagiarism.urkund.UrkundSubmissionRepository;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;

class PlagiarismControlImpl implements PlagiarismControl {
    private final PlagiarismRequestRepository plagiarismRequestRepository;
    private final UrkundSubmissionRepository urkundSubmissionRepository;

    @Inject
    public PlagiarismControlImpl(final PlagiarismRequestRepository plagiarismRequestRepository, final UrkundSubmissionRepository urkundSubmissionRepository) {
        this.plagiarismRequestRepository = plagiarismRequestRepository;
        this.urkundSubmissionRepository = urkundSubmissionRepository;
    }

    @Override
    public void submit(final FileDescription file, final User receiver) {
        final PlagiarismRequest plagiarismRequest = new PlagiarismRequest();
        plagiarismRequest.setFileDescription(file);
        plagiarismRequest.setReceiver(receiver);
        plagiarismRequestRepository.save(plagiarismRequest);
    }

    @Override
    public boolean isQueued(final FileDescription file) {
        return plagiarismRequestRepository.findByFileDescription(file).isPresent();
    }

    @Override
    @Transactional
    public void resubmit(final FileDescription file, final User receiver) {
        urkundSubmissionRepository.findByFile(file)
                .ifPresent(urkundSubmissionRepository::delete);
        submit(file, receiver);
    }
}
