package se.su.dsv.scipro.report;

import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.calculators.original.*;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Calendar;

public class GradeCalculatorServiceImpl implements GradeCalculatorService {
    @Override
    public GradeCalculator getSupervisorCalculator(final Project project) {
        DegreeLevel degreeLevel = project.getProjectType().getDegreeLevel();
        if (degreeLevel.equals(DegreeLevel.FIRST_CYCLE)) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.SupervisorBachelorGradeCalculator();
            }
            else {
                return new SupervisorBachelorGradeCalculator();
            }
        } else if (degreeLevel.equals(DegreeLevel.SECOND_CYCLE)) {
            return getMasterGradeCalculator(project);
        }
        throw new IllegalStateException("No grade calculation implemented for project type");
    }

    private GradeCalculator getMasterGradeCalculator(final Project project) {
        int credits = project.getCredits();
        if (credits == ProjectType.HP_15) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.SupervisorMagisterGradeCalculator();
            }
            else {
                return new SupervisorMaster15GradeCalculator();
            }
        } else if (credits == ProjectType.HP_30 || credits == 0) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.SupervisorMasterGradeCalculator();
            }
            else {
                return new SupervisorMaster30GradeCalculator();
            }
        }
        throw new IllegalStateException("No grade calculation implemented for master project with " + credits + " credits");
    }

    @Override
    public GradeCalculator getReviewerCalculator(final SupervisorGradingReport supervisorGradingReport) {
        final Project project = supervisorGradingReport.getProject();
        DegreeLevel degreeLevel = project.getProjectType().getDegreeLevel();
        if (degreeLevel.equals(DegreeLevel.FIRST_CYCLE)) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.ReviewerBachelorGradeCalculator(supervisorGradingReport);
            }
            else {
                return new ReviewerBachelorGradeCalculator(supervisorGradingReport);
            }
        } else if (degreeLevel.equals(DegreeLevel.SECOND_CYCLE)) {
            return getMasterGradeCalculator(supervisorGradingReport);
        }
        throw new IllegalStateException("No grade calculation implemented for project type");
    }

    private GradeCalculator getMasterGradeCalculator(SupervisorGradingReport supervisorGradingReport) {
        final Project project = supervisorGradingReport.getProject();
        int credits = project.getCredits();
        if (credits == ProjectType.HP_15) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.ReviewerMagisterGradeCalculator(supervisorGradingReport);
            } else {
                return new ReviewerMaster15GradeCalculator(supervisorGradingReport);
            }
        } else if (credits == ProjectType.HP_30 || credits == 0) {
            if (getYear(project) >= 2017) {
                return new se.su.dsv.scipro.report.calculators.vt17.ReviewerMasterGradeCalculator(supervisorGradingReport);
            } else {
                return new ReviewerMaster30GradeCalculator(supervisorGradingReport);
            }
        }
        throw new IllegalStateException("No grade calculation implemented for master project with " + credits + " credits");
    }

    private int getYear(final Project project) {
        final Calendar calendar = Calendar.getInstance();
        calendar.setTime(project.getDateCreated());
        return calendar.get(Calendar.YEAR);
    }
}
