package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import com.querydsl.core.types.EntityPath;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.util.Either;

import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;

abstract class AbstractReviewerApprovalServiceImpl<T extends ReviewerApproval> extends AbstractServiceImpl<T, Long> implements ReviewerApprovalService<T> {

    protected final FileDescriptionService fileDescriptionService;
    protected final EventBus eventBus;

    AbstractReviewerApprovalServiceImpl(Provider<EntityManager> em, Class<T> clazz, EntityPath<T> path, final EventBus eventBus, final FileDescriptionService fileDescriptionService) {
        super(em, clazz, path);
        this.eventBus = eventBus;
        this.fileDescriptionService = fileDescriptionService;
    }

    @Override
    public boolean requiresUpload(final Project project) {
        return findBy(project).forall(ReviewerApproval::isRejected);
    }

    protected Either<AlreadyRequested, T> addNewRequest(final String comment, final FileDescription storedThesis, final T earlier) {
        final Decision currentDecision = earlier.getCurrentDecision();
        if (currentDecision.getStatus() == Status.REJECTED) {
            final Date deadline = getDeadline(new Date());
            earlier.addNewThesis(storedThesis, comment, deadline);
            return Either.right(earlier);
        }
        else {
            return Either.left(new AlreadyRequested(currentDecision));
        }
    }

    protected abstract Date getDeadline(Date requested);
}
