package se.su.dsv.scipro.reviewing;

import com.querydsl.core.types.SubQueryExpression;
import com.querydsl.jpa.JPAExpressions;
import se.su.dsv.scipro.system.AbstractRepository;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.querydsl.core.types.dsl.Expressions.allOf;
import static com.querydsl.core.types.dsl.Expressions.anyOf;

class ReviewerDeadlineFollowupServiceImpl extends AbstractRepository implements ReviewerDeadlineFollowupService {

    @Inject
    ReviewerDeadlineFollowupServiceImpl(final Provider<EntityManager> em) {
        super(em);
    }

    @Override
    public List<Pair<? extends ReviewerApproval, Decision>> getLateReviewerApprovals(ReviewerApproval.Step step, Date startDate, Date endDate, final Date now) {
        List<Decision> decisions = from(QDecision.decision)
                .where(allOf(QDecision.decision.requested.after(startDate),
                        QDecision.decision.requested.before(endDate),
                        QDecision.decision.reviewerApproval.id.in(getSubQueryList(step)),
                        anyOf(
                                allOf(QDecision.decision.deadline.before(now), QDecision.decision.decisionDate.isNull()),
                                QDecision.decision.decisionDate.after(QDecision.decision.deadline))
                        )
                )
                .fetch();
        return getLateDecisions(decisions);
    }

    private SubQueryExpression<Long> getSubQueryList(ReviewerApproval.Step step) {
        if (step.equals(ReviewerApproval.Step.ROUGH_DRAFT_APPROVAL))
            return JPAExpressions.select(QRoughDraftApproval.roughDraftApproval.id)
                    .from(QRoughDraftApproval.roughDraftApproval);
        return JPAExpressions.select(QFinalSeminarApproval.finalSeminarApproval.id)
                .from(QFinalSeminarApproval.finalSeminarApproval);
    }

    private List<Pair<? extends ReviewerApproval, Decision>> getLateDecisions(final List<Decision> decisions) {
        final List<Pair<? extends ReviewerApproval, Decision>> late = new ArrayList<>();
        for (final Decision decision : decisions) {
            boolean isLate = decision.isLate();
            if (isLate) {
                late.add(new Pair<>(decision.getReviewerApproval(), decision));
            }
        }
        return late;
    }
}
