package se.su.dsv.scipro.system;

import com.querydsl.core.types.dsl.BooleanExpression;
import se.su.dsv.scipro.project.QProject;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.List;

import static com.querydsl.core.types.dsl.Expressions.allOf;
import static com.querydsl.core.types.dsl.Expressions.anyOf;

public class ProjectTypeServiceImpl extends AbstractServiceImpl<ProjectType,Long> implements ProjectTypeService {

    @Inject
    public ProjectTypeServiceImpl(Provider<EntityManager> em) {
        super(em, ProjectType.class, QProjectType.projectType);
    }

    @Override
    public List<ProjectType> findAllActive() {
        return findAll(notDeleted());
    }

    @Override
    public List<ProjectType> findAllDeleted() {
        return findAll(QProjectType.projectType.deleted.isTrue());
    }

    @Override
    public List<ProjectType> findByDegreeLevels(Collection<DegreeLevel> degreeLevels) {
        return findAll(allOf(QProjectType.projectType.degreeLevel.in(degreeLevels), notDeleted()));
    }

    @Override
    public List<ProjectType> findWithModule(ProjectModule projectModule) {
        return findAll(allOf(QProjectType.projectType.projectModules.any().eq(projectModule), notDeleted()));
    }

    @Override
    public List<ProjectType> findBySupervisorProjects(User supervisor) {
        final QProject project = QProject.project;
        return from(project)
                .select(project.projectType)
                .distinct()
                .where(anyOf(project.headSupervisor.eq(supervisor),
                        project.reviewers.any().eq(supervisor),
                        project.coSupervisors.any().eq(supervisor)))
                .fetch();
    }

    @Override
    public boolean projectTypeWithModuleExists(ProjectModule module) {
        return !findWithModule(module).isEmpty();
    }

    private static BooleanExpression notDeleted() {
        return QProjectType.projectType.deleted.isFalse();
    }
}
