package se.su.dsv.scipro.workerthreads;

import se.su.dsv.scipro.mail.MailEventWorker;
import se.su.dsv.scipro.plagiarism.PlagiarismSubmitter;
import se.su.dsv.scipro.plagiarism.urkund.StatusPollingWorker;
import se.su.dsv.scipro.reviewing.ReviewerDecisionReminderWorker;

import javax.inject.Inject;
import javax.inject.Provider;
import java.util.concurrent.TimeUnit;

class TemporaryWorkerScheduler {
    @Inject
    TemporaryWorkerScheduler(
            Scheduler scheduler,
            Provider<MailEventWorker> mailEventWorker,
            Provider<NotificationCompilationWorker> notificationCompilationWorker,
            Provider<IdeaExportWorker> ideaExportWorker,
            Provider<ThesisUploadReminderWorker> thesisUploadReminderWorker,
            Provider<ThesisUploadDeadlineWorker> thesisUploadDeadlineWorker,
            Provider<ManualMatchRemindWorker> manualMatchRemindWorkerProvider,
            Provider<AnnualReviewWorker> annualReviewWorker,
            Provider<ReviewerDecisionReminderWorker> reviewerDecisionReminderWorker,
            Provider<PlagiarismSubmitter> plagiarismSubmitter,
            Provider<StatusPollingWorker> urkundPoller)
    {
        scheduler.schedule("Mail-event-worker").runBy(mailEventWorker).every(1, TimeUnit.MINUTES);
        scheduler.schedule("Check-plagiarism-event-worker").runBy(plagiarismSubmitter).every(10, TimeUnit.MINUTES);
        scheduler.schedule("Send out summary mails for all unread notifications").runBy(notificationCompilationWorker).dailyAt(15, 0);
        scheduler.schedule("Export eligible ideas").runBy(ideaExportWorker).dailyAt(1, 0);
        scheduler.schedule("Warn about expiring thesis upload deadline").runBy(thesisUploadReminderWorker).dailyAt(4, 0);
        scheduler.schedule("Final seminar upload deadline cancel worker").runBy(thesisUploadDeadlineWorker).dailyAt(4, 0);
        scheduler.schedule("Warn about more than one year passed from the latest uploaded annual review").runBy(annualReviewWorker).dailyAt(0, 1);
        scheduler.schedule("Remind match responsible to manually match ideas").runBy(manualMatchRemindWorkerProvider).dailyAt(4, 0);
        scheduler.schedule("Reviewer decision reminders").runBy(reviewerDecisionReminderWorker).dailyAt(11, 45);
        scheduler.schedule("Poll URKUND for status").runBy(urkundPoller).every(15, TimeUnit.MINUTES);
    }
}
