package se.su.dsv.scipro.grading

import javax.inject.Provider
import javax.persistence.EntityManager

import com.querydsl.core.types.dsl.{Expressions, StringPath}
import com.querydsl.core.types.{EntityPath, PathMetadataFactory}
import com.querydsl.jpa.impl.JPAQuery
import org.springframework.data.domain.{Pageable, Sort}

trait FindAll[T] {
  
  val em: Provider[EntityManager]
  
  val path: EntityPath[T]

  private def createQuery: JPAQuery[T] = {
    from(path)
  }

  private def from(paths: EntityPath[T]*): JPAQuery[T] = {
    new JPAQuery(em.get).from(paths:_*)
  }
  
  private def pagedQuery(pageable: Pageable): JPAQuery[T] = {
    applyPageable(createQuery, pageable)
  }

  private def applyPageable(query: JPAQuery[T], pageable: Pageable): JPAQuery[T] = {
    val pagedQuery: JPAQuery[T] = paged(query, pageable.getOffset, pageable.getLimit)
    if (pageable.getSort == null) {
      pagedQuery
    } else {
      sorted(pagedQuery, pageable.getSort)  
    }
  }

  private def paged(query: JPAQuery[T], offset: Long, limit: Long): JPAQuery[T] = {
    query.limit(limit).offset(offset)
  }

  private def sorted(query: JPAQuery[T], sort: Sort): JPAQuery[T] = {
    val property: StringPath = Expressions.stringPath(path, sort.getProperty)
    query.orderBy(if (sort.isAscending) property.asc else property.desc)
  }

  def findAll(pageable: Pageable): Iterable[T] = {
    import scala.collection.JavaConverters._
    pagedQuery(pageable).fetch().asScala
  }
}
