package se.su.dsv.scipro.integration.daisy;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.daisyExternal.http.DaisyAPI;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserSearchService;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import javax.ws.rs.WebApplicationException;
import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

class DaisyUserSearchService implements UserSearchService {
    private static final Logger LOGGER = LoggerFactory.getLogger(DaisyUserSearchService.class);

    private final DaisyAPI daisyAPI;
    private final UserService userService;

    @Inject
    DaisyUserSearchService(final DaisyAPI daisyAPI, final UserService userService) {
        this.daisyAPI = daisyAPI;
        this.userService = userService;
    }

    @Override
    public Collection<User> search(final String term, final Roles role, final long maxResults) {
        if (term != null && term.length() >= 6) {
            try {
                return daisyAPI.findByPersonnummer(term).stream()
                        .map(person -> userService.findByExternalIdentifier(person.getId()))
                        .filter(User.class::isInstance)
                        .filter(user -> user.hasRole(role))
                        .collect(Collectors.toList());
            }
            catch (WebApplicationException e) {
                LOGGER.debug("Failed to search for users in Daisy", e);
            }
        }
        return Collections.emptyList();
    }
}
