package se.su.dsv.scipro.components;

import org.apache.wicket.AttributeModifier;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;

import java.util.List;

public abstract class BootstrapTabs<T> extends GenericPanel<List<T>> {

    private static final AttributeAppender ACTIVE = AttributeAppender.append("class", "active");

    protected BootstrapTabs(final String id, final IModel<List<T>> ts) {
        super(id, ts);

        add(new ListView<T>("tabs", ts) {
            @Override
            protected void populateItem(final ListItem<T> item) {
                final IModel<String> labelModel = getLabelModel(item.getModel());
                final String href = "#" + BootstrapTabs.this.getHref(labelModel);
                final ExternalLink tab = new ExternalLink("tabLink", Model.of(href), labelModel);
                if (item.getIndex() == 0) {
                    item.add(ACTIVE);
                }
                item.add(tab);
            }
        });

        add(new ListView<T>("content", ts) {
            {
                setReuseItems(true);
            }

            @Override
            protected void populateItem(final ListItem<T> item) {
                final WebMarkupContainer wmc = getContent("id", item.getModel());
                final IModel<String> labelModel = getLabelModel(item.getModel());
                final String href = BootstrapTabs.this.getHref(labelModel);
                if (item.getIndex() == 0) {
                    item.add(ACTIVE);
                }
                item.add(AttributeModifier.replace("id", href));
                item.add(wmc);
            }
        });
    }

    private String getHref(final IModel<String> labelModel) {
        return labelModel.getObject().toLowerCase().replaceAll("[^a-z]", "");
    }

    protected abstract WebMarkupContainer getContent(final String id, final IModel<T> model);

    protected abstract IModel<String> getLabelModel(final IModel<T> model);
}
