package se.su.dsv.scipro.files;

import lombok.SneakyThrows;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.Option;

import java.io.IOException;
import java.io.InputStream;
import java.util.Objects;
import java.util.function.Function;

public class WicketFileUpload implements FileUpload {
    private final org.apache.wicket.markup.html.form.upload.FileUpload fileUpload;
    private final User user;

    public WicketFileUpload(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
        this.fileUpload = Objects.requireNonNull(fileUpload, "fileUpload may not be null");
        this.user = SciProSession.get().getUser();
    }

    public String getFileName() {
        return fileUpload.getClientFileName();
    }

    public String getContentType() {
        return fileUpload.getContentType();
    }

    public User getUploader() {
        return user;
    }

    @Override
    public long getSize() {
        return fileUpload.getSize();
    }

    @Override
    @SneakyThrows(IOException.class)
    public <T> T handleData(Function<InputStream, T> handler) {
        try (InputStream is = fileUpload.getInputStream()) {
            return handler.apply(is);
        }
    }

    public static FileUpload from(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
        return new WicketFileUpload(fileUpload);
    }

    public static scala.Option<FileUpload> ofOptional(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
        return scala.Option.apply(fileUpload).map(new AbstractFunction1<org.apache.wicket.markup.html.form.upload.FileUpload, FileUpload>() {
            @Override
            public FileUpload apply(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
                return new WicketFileUpload(fileUpload);
            }
        });
    }

    public static Option<FileUpload> ofOption(org.apache.wicket.markup.html.form.upload.FileUpload fileUpload) {
        return Option.of(fileUpload).map(WicketFileUpload::new);
    }
}
