package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.Component;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.link.StatelessLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.Either;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.text.SimpleDateFormat;
import java.util.Date;

public abstract class OpposeColumnPanel extends Panel {

    public static final String LINK = "link";
    public static final String OPPOSE_CONFIRMATION = "Are you sure you want to oppose on this seminar?";

    @Inject
    private FinalSeminarService finalSeminarService;

    public abstract void onClick(IModel<FinalSeminar> clickedModel);

    public OpposeColumnPanel(String id, final IModel<FinalSeminar> finalSeminarModel, final IModel<Project> projectModel) {
        super(id, finalSeminarModel);

        final Either<OppositionRegistrationStatus, Void> canOppose =
                finalSeminarService.canOppose(SciProSession.get().getUser(), finalSeminarModel.getObject(), projectModel.getObject());
        final Component opposeLink = canOppose.fold(
                notAllowed -> showError(notAllowed),
                allowed -> new OpposeLink(LINK, finalSeminarModel)
        );

        add(opposeLink);
    }

    private Component showError(OppositionRegistrationStatus notAllowed) {
        return notAllowed.fold(
                ungradedOpposition -> getLabel("You have ungraded oppositions"),
                finalSeminarCancelled -> getLabel("The final seminar has been cancelled"),
                alreadyParticipating -> getLabel("You are already participating on the final seminar"),
                alreadyHappened -> getLabel("The seminar has already happened"),
                tooManyOpponents -> getLabel("The seminar is already full on opponents"),
                priorityForSeminarAuthors -> getLabel("Authors with their own final seminar have " +
                        "priority to register for " + priorityForSeminarAuthors.getPriorityDays() +
                        " day(s). This priority will end at " + asDateTime(Date.from(priorityForSeminarAuthors.getPriorityEnd()))),
                alreadyOpposed -> getLabel("You have already completed an opposition and can not perform a second one."));
    }

    public String asDateTime(Date date) {
        SimpleDateFormat dateFormat = new SimpleDateFormat(DateStyle.DATETIME.getFormat());
        return dateFormat.format(date);
    }

    private Component getLabel(final String label) {
        // Dumb workaround to get the <a> tag transformed into a <span>
        return new StatelessLink<Void>(LINK){
            @Override
            public void onClick() {}
        }
                .setBody(Model.of(label))
                .setEnabled(false);
    }

    private class OpposeLink extends Link<FinalSeminar> {

        private final IModel<FinalSeminar> finalSeminarModel;

        public OpposeLink(final String id, final IModel<FinalSeminar> finalSeminarModel) {
            super(id);
            add(new JavascriptEventConfirmation("onclick", OpposeColumnPanel.OPPOSE_CONFIRMATION));
            this.finalSeminarModel = finalSeminarModel;
            setOutputMarkupId(true);
        }

        @Override
        public void onClick() {
            OpposeColumnPanel.this.onClick(finalSeminarModel);
        }
    }
}
