package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.List;
import java.util.stream.Collectors;

public class ProjectOppositionsListPanel extends GenericPanel<Project> {
    @Inject
    private FinalSeminarService finalSeminarService;

    public ProjectOppositionsListPanel(final String id, final IModel<Project> model) {
        super(id, model);

        add(new AutoHidingListView<FinalSeminarOpposition>("oppositions", getOpponents(getSeminars())) {
            @Override
            protected void populateItem(final ListItem<FinalSeminarOpposition> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), item.getModelObject().getFinalSeminar().getProject().getId());

                BookmarkablePageLink<Void> finalSeminarLink = new BookmarkablePageLink<>("finalSeminar", ProjectFinalSeminarDetailsPage.class, pp);
                finalSeminarLink.add(new DateLabel("finalSeminar.startDate", DateStyle.DATE));
                item.add(finalSeminarLink);
                item.add(new UserLinkPanel("user"));
                item.add(new Label("points") {
                    @Override
                    protected void onConfigure() {
                        setVisible(getDefaultModelObject() != null);
                    }
                });
            }

            @Override
            protected IModel<FinalSeminarOpposition> getListItemModel(final IModel<? extends List<FinalSeminarOpposition>> listViewModel, final int index) {
                return new CompoundPropertyModel<>(super.getListItemModel(listViewModel, index));
            }
        });

        add(new Label("no.oppositions", new ResourceModel("no.oppositions"))
                .add(new OppositeVisibility(get("oppositions"))));
    }

    private IModel<? extends List<FinalSeminarOpposition>> getOpponents(final IModel<List<FinalSeminar>> seminars) {
        return new LoadableDetachableModel<List<FinalSeminarOpposition>>() {
            @Override
            protected List<FinalSeminarOpposition> load() {
                return seminars.getObject().stream()
                        .flatMap(input -> input.getOppositions().stream())
                        .filter(input -> getModelObject().isParticipant(input.getUser()))
                        .collect(Collectors.toList());
            }

            @Override
            protected void onDetach() {
                seminars.detach();
            }
        };
    }

    private IModel<List<FinalSeminar>> getSeminars() {
        return new LoadableDetachableModel<List<FinalSeminar>>() {
            @Override
            protected List<FinalSeminar> load() {
                return finalSeminarService.findProjectOpposing(getModelObject(), new PageRequest(0, Integer.MAX_VALUE));
            }
        };
    }
}
