package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.BootstrapDateField;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.StatelessModel;
import se.su.dsv.scipro.data.FilteredListModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.oauth.OAuth;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

abstract class AbstractExaminationsPanel extends GenericPanel<User> {

    @Inject
    private GradingService gradingService;

    private final IModel<Project> projectModel;

    public AbstractExaminationsPanel(
            final String id,
            final IModel<Project> projectModel,
            final IModel<User> userModel,
            final IModel<List<Examination>> examinations)
    {
        super(id, userModel);
        this.projectModel = projectModel;

        add(new ListView<Examination>("examinations", examinations) {
            {
                setReuseItems(true);
            }

            @Override
            protected void populateItem(final ListItem<Examination> item) {
                item.add(new ReportingForm("examination", item.getModel(), userModel));
            }
        });
    }

    private class ReportingForm extends StatelessForm<Examination> {
        private final IModel<User> userModel;
        private final IModel<Grade> gradeModel = new StatelessModel<>();
        private final IModel<Date> reportedModel = new StatelessModel<>();

        public ReportingForm(
                final String id,
                final IModel<Examination> examinationModel,
                final IModel<User> userModel)
        {
            super(id, examinationModel);
            this.userModel = userModel;

            final IModel<Result> resultModel = new LoadableDetachableModel<Result>() {
                @Override
                protected Result load() {
                    return gradingService.getResult(
                            getSession().getMetaData(OAuth.TOKEN),
                            projectModel.getObject().getIdentifier(),
                            userModel.getObject().getIdentifier(),
                            examinationModel.getObject().getId())
                            .orElse(null);
                }
            };

            queue(new WebMarkupContainer("resultContainer") {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(resultModel.getObject() != null);
                }
            });
            queue(new Label("examinationName1", new PropertyModel<>(examinationModel, "name.english")));
            queue(new Label("letter", new PropertyModel<>(resultModel, "letter")));
            queue(new DateLabel("reported", new PropertyModel<>(resultModel, "reported"), DateStyle.DATE));

            queue(new WebMarkupContainer("reportingContainer") {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(resultModel.getObject() == null);
                }
            });
            queue(new Label("examinationName2", new PropertyModel<>(examinationModel, "name.english")));
            final IModel<List<Grade>> grades = new PropertyModel<>(examinationModel, "grades");
            final IModel<List<Grade>> passing = new FilteredListModel<Grade>(grades) {
                @Override
                protected boolean accept(final Grade grade) {
                    return grade.getType() == Grade.Type.PASSING;
                }
            };
            final DropDownChoice<Grade> gradeField = new DropDownChoice<>(
                    "gradeField",
                    gradeModel,
                    passing,
                    new ChoiceRenderer<>("letter", "letter"));
            gradeField.setRequired(true);
            queue(gradeField);
            final BootstrapDateField reportedField = new BootstrapDateField(
                    "reportedField",
                    reportedModel);
            reportedField.setRequired(true);
            queue(reportedField);
        }

        @Override
        protected void onSubmit() {
            reportGrade(
                    projectModel.getObject(),
                    userModel.getObject(),
                    getModelObject(),
                    gradeModel.getObject().getLetter(),
                    reportedModel.getObject());
        }
    }

    protected abstract void reportGrade(
            final Project project,
            final User user,
            final Examination examination,
            final String letter,
            final Date reported);
}
