package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.finalthesis.ApproveFinalThesisPanel;
import se.su.dsv.scipro.forum.panels.threaded.ViewForumThreadPanel;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.reviewer.ReviewerInteractionForumThread;
import se.su.dsv.scipro.reviewing.ReviewerInteractionService;
import se.su.dsv.scipro.security.auth.ProjectModuleComponent;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.supervisor.pages.AbstractSupervisorProjectDetailsPage;
import se.su.dsv.scipro.system.ProjectModule;

import javax.inject.Inject;

@ProjectModuleComponent(ProjectModule.GRADING)
public class SupervisorGradingReportPage extends AbstractSupervisorProjectDetailsPage {

    public static final String FILL_OUT_PANEL = "fillOutPanel";

    @Inject
    private ReviewerInteractionService reviewerInteractionService;
    @Inject
    private FileDescriptionService fileDescriptionService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    public SupervisorGradingReportPage(PageParameters pp) {
        super(pp);
        add(new ApproveFinalThesisPanel("finalThesis", projectModel));
        add(new UploadTextMatchingPanel("textMatching", projectModel));

        final String externalGradingURL = generalSystemSettingsService.getGeneralSystemSettingsInstance().getExternalGradingURL();
        add(new ExternalLink("moreInformation", externalGradingURL) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(getDefaultModelObject() != null);
            }
        });

        add(new FillOutGradingReportPanel(FILL_OUT_PANEL, projectModel) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(projectModel.getObject().getHeadSupervisor().equals(SciProSession.get().getUser()));
            }
        });

        add(new ViewForumThreadPanel<>("discussion", projectModel, new ReviewerInteractionForumThread(reviewerInteractionService, fileDescriptionService)));
    }
}
