package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;

public class ThesisApprovedPanel extends GenericPanel<Project> {
    private static final String FILE_PANEL = "filePanel";

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private GradingReportService gradingReportService;

    public ThesisApprovedPanel(String id, final IModel<Project> project) {
        super(id, project);
        add(new ViewAttachmentPanel(FILE_PANEL, getFileDescription(), DateStyle.DATETIME));
        add(new Link<Project>("reject", project) {
            {
                add(new JavascriptEventConfirmation("onclick", new ResourceModel("remove.approved")));
            }

            @Override
            public void onClick() {
                finalThesisService.reject(getModelObject());
            }
        });
    }

    @Override
    protected void onConfigure() {
        setVisibilityAllowed(gradingReportTemplateIsAvailable());
    }

    private boolean gradingReportTemplateIsAvailable() {
        return gradingReportService.getTemplate(getModelObject()) != null;
    }

    private LoadableDetachableModel<FileDescription> getFileDescription() {
        return new LoadableDetachableModel<FileDescription>() {
            @Override
            protected FileDescription load() {
                return finalThesisService.getFinalThesisFileDescription(getModelObject());
            }
        };
    }
}
