package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.StatelessForm;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.upload.FileUpload;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.finalthesis.FinalThesis;
import se.su.dsv.scipro.finalthesis.FinalThesisService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.report.GradingReportService;
import se.su.dsv.scipro.report.SupervisorGradingReport;
import se.su.dsv.scipro.repository.panels.ViewAttachmentPanel;

import javax.inject.Inject;
import java.util.List;

public class UploadTextMatchingPanel extends GenericPanel<Project> {

    @Inject
    private FinalThesisService finalThesisService;
    @Inject
    private GradingReportService gradingReportService;
    @Inject
    private FileDescriptionService fileDescriptionService;

    public UploadTextMatchingPanel(final String id, final IModel<Project> projectModel) {
        super(id, projectModel);

        final IModel<FinalThesis> finalThesisModel = new LoadableDetachableModel<FinalThesis>() {
            @Override
            protected FinalThesis load() {
                return finalThesisService.findByProject(getModelObject());
            }
        };

        final FileUploadField textMatchingDocumentField =
                new FileUploadField("textMatchingDocument");
        textMatchingDocumentField.setRequired(true);

        final TextArea<String> commentField =
                new TextArea<>("comment", new Model<>());
        commentField.setRequired(true);

        add(new StatelessForm<FinalThesis>("form", finalThesisModel) {
            {
                add(textMatchingDocumentField);
                add(commentField);
            }

            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getModelObject().getTextMatchingDocument() == null);
            }

            @Override
            protected void onSubmit() {
                final String comment = commentField.getModelObject();
                final List<SupervisorGradingReport> supervisorGradingReports =
                        gradingReportService.getSupervisorGradingReports(projectModel.getObject());
                supervisorGradingReports.forEach(report -> {
                    report.setPlagiarismFeedback(comment);
                    gradingReportService.save(report);
                });

                final FileUpload fileUpload =
                        textMatchingDocumentField.getFileUpload();
                final FileDescription textMatchingDocument =
                        fileDescriptionService.storeFile(new WicketFileUpload(fileUpload));
                final FinalThesis finalThesis = getModelObject();
                finalThesis.setTextMatchingDocument(textMatchingDocument);
                finalThesisService.save(finalThesis);
            }
        });

        add(new WebMarkupContainer("alreadyUploaded") {
            {
                add(new ViewAttachmentPanel(
                        "textMatchingDocument",
                        new PropertyModel<>(finalThesisModel, "textMatchingDocument")));
            }
        });
    }

    @Override
    protected void onConfigure() {
        final FinalThesis finalThesis = finalThesisService.findByProject(getModelObject());
        setVisibilityAllowed(finalThesis != null && finalThesis.getStatus() == FinalThesis.Status.APPROVED);
    }
}
