package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.link.Link;
import se.su.dsv.scipro.basepages.BasePage;
import se.su.dsv.scipro.mail.MailEvent;
import se.su.dsv.scipro.mail.MailEventService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.JavascriptEventConfirmation;

import javax.inject.Inject;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

@Authorization(authorizedRoles = {Roles.ADMIN, Roles.SYSADMIN})
public class AdminIdeaPopupPage extends BasePage {

    public static final String IDEA_PANEL = "ideaPanel";

    @Inject
    private PreliminaryMatchService preliminaryMatchService;
    @Inject
    private IdeaService ideaService;
    @Inject
    private MailEventService mailEventService;

    public AdminIdeaPopupPage() {
        final AdminIdeaPopupPanel ideaPopupPanel = new AdminIdeaPopupPanel(IDEA_PANEL);
        add(ideaPopupPanel);

        add(new Link<Void>("sendStatus") {
            {
                add(new JavascriptEventConfirmation("onclick", "Are you sure you want to send the status e-mails?"));
            }

            @Override
            public void onClick() {
                final IdeaService.Filter filter = ideaPopupPanel.getFilter();

                if (filter.getApplicationPeriod() == null) {
                    error("Please select an application period in the filter");
                    return;
                }
                ideaService.findAll(filter)
                        .stream()
                        .filter(idea -> idea.getMatchStatus() == Idea.Status.MATCHED)
                        .collect(Collectors.groupingBy(idea -> idea.getMatch().getSupervisor()))
                        .entrySet()
                        .forEach(entry -> {
                            final User supervisor = entry.getKey();
                            final List<Idea> ideas = entry.getValue();

                            final StringBuilder message = new StringBuilder();
                            message.append("You have been matched to ")
                                    .append(ideas.size())
                                    .append(" ideas for the period ")
                                    .append(filter.getApplicationPeriod().getName())
                                    .append("\n\n")
                                    .append("Now would be a good time to schedule the first meeting if you have yet not done so.")
                                    .append("\n\n")
                                    .append("Here are the ideas you've been matched to:")
                                    .append("\n");
                            ideas.forEach(idea -> {
                                message.append("* ")
                                        .append(idea.getTitle())
                                        .append(" (")
                                        .append(idea.getProjectType().getName())
                                        .append(")")
                                        .append("\n");
                                idea.getIdeaParticipations()
                                        .forEach(author -> {
                                            message.append("   - ")
                                                    .append(author.getUser().getFullName())
                                                    .append(" <")
                                                    .append(author.getUser().getEmailAddress())
                                                    .append(">")
                                                    .append("\n");
                                        });
                            });

                            final MailEvent mailEvent = new MailEvent();
                            mailEvent.setRecipients(Collections.singleton(supervisor));
                            mailEvent.setSubject("Matched ideas for the period " + filter.getApplicationPeriod().getName());
                            mailEvent.setMessageBody(message.toString());
                            mailEventService.save(mailEvent);
                        });
                success("E-mail sent");
            }
        });
        add(new Link<Void>("complete") {
            @Override
            public void onClick() {
                preliminaryMatchService.completePreliminaryMatch(ideaPopupPanel.getFilter());
            }
        });
    }
}
