package se.su.dsv.scipro.match;

import org.apache.commons.lang3.StringUtils;
import org.apache.wicket.Component;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.AjaxFormComponentUpdatingBehavior;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.PropertyColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.EmptyPanel;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.*;
import se.su.dsv.scipro.components.EmployeeAutoComplete;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

public class AdminIdeaPopupPanel extends AbstractAdminIdeaPanel {

    @Inject
    private PreliminaryMatchService preliminaryMatchService;

    public AdminIdeaPopupPanel(String id) {
        super(id);
    }

    @Override
    protected List<IColumn<Idea, String>> addPanelSpecificColumns() {
        List<IColumn<Idea, String>> list = new ArrayList<>();
        list.add(new PropertyColumn<Idea, String>(Model.of("Supervisor unit"), "match.supervisor.unit.title"));
        list.add(new AbstractColumn<Idea, String>(Model.of("Preliminary match")) {
            @Override
            public void populateItem(final Item<ICellPopulator<Idea>> cellItem, final String componentId, final IModel<Idea> rowModel) {
                if (rowModel.getObject().getType() == Idea.Type.STUDENT) {
                    final IModel<PreliminaryMatch> preliminaryMatch = getPreliminaryMatch(rowModel);
                    cellItem.add(new PreliminaryCell(componentId, preliminaryMatch));
                }
                else {
                    cellItem.add(new EmptyPanel(componentId));
                }
            }
        });
        list.add(new AbstractExportableColumn<Idea, String>(Model.of("\"Student idea form\"")) {
            @Override
            protected Component createDisplayComponent(final String componentId, final IModel<?> dataModel) {
                return new MultiLineLabel(componentId, dataModel);
            }

            @Override
            public IModel<String> getDataModel(IModel<Idea> rowModel) {
                return new AbstractReadOnlyModel<String>() {
                    private final StringBuilder stringBuilder = new StringBuilder();

                    /* constructor */ {
                        appendIfNotEmpty("BACKGROUND", TholanderBox::getBackground);
                        appendIfNotEmpty("LITERATURE", TholanderBox::getLiterature);
                        appendIfNotEmpty("PROBLEM", TholanderBox::getProblem);
                        appendIfNotEmpty("METHOD", TholanderBox::getMethod);
                        appendIfNotEmpty("INTERESTS", TholanderBox::getInterests);
                    }

                    private void appendIfNotEmpty(String label, Function<TholanderBox, String> f) {
                        final String s = f.apply(rowModel.getObject().getTholanderBox());
                        if (StringUtils.isNotEmpty(s)) {
                            stringBuilder.append(label).append(": ").append(s).append("\n\n");
                        }
                    }

                    @Override
                    public String getObject() {
                        return stringBuilder.toString();
                    }
                };
            }
        });
        list.add(new PropertyColumn<Idea, String>(Model.of("Research area"), "researchArea.title"));
        list.add(new KeywordsColumn<String>(Model.of("Keywords")));
        list.add(new PropertyColumn<Idea, String>(Model.of("Description"), "description"));
        list.add(new PropertyColumn<Idea, String>(Model.of("Prerequisites"), "prerequisites"));
        return list;
    }

    private IModel<PreliminaryMatch> getPreliminaryMatch(final IModel<Idea> rowModel) {
        return new LoadableDetachableModel<PreliminaryMatch>() {
            @Override
            protected PreliminaryMatch load() {
                final Idea idea = rowModel.getObject();
                final PreliminaryMatch preliminaryMatch
                        = preliminaryMatchService.findByIdea(idea);
                return preliminaryMatch == null
                        ? new PreliminaryMatch(idea)
                        : preliminaryMatch;
            }
        };
    }

    private class PreliminaryCell extends Panel {
        public PreliminaryCell(final String id, final IModel<PreliminaryMatch> model) {
            super(id, model);
            add(new EmployeeAutoComplete("supervisor", new PropertyModel<>(model, "supervisor")) {
                @Override
                protected void onInitialize() {
                    super.onInitialize();
                    getSettings().setPlaceholder("Supervisor");
                }

                @Override
                protected void action(final AjaxRequestTarget target, final User object) {
                    preliminaryMatchService.save(model.getObject());
                }

                @Override
                public boolean getAllowClear() {
                    return true;
                }
            });
            final AjaxFormComponentUpdatingBehavior saveOnChange = new AjaxFormComponentUpdatingBehavior("input") {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    preliminaryMatchService.save(model.getObject());
                }
            };
            final TextArea<String> comment = new TextArea<>("comment", new PropertyModel<>(model, "comment"));
            comment.add(saveOnChange);
            add(comment);
        }
    }
}
