package se.su.dsv.scipro.match;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.util.string.StringValue;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightProjectIdeas;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.Date;

@Authorization(authorizedRoles={Roles.AUTHOR})
public class ProjectIdeaSubmissionPage extends AbstractIdeaProjectPage implements MenuHighlightProjectIdeas {
    public static final String SUBMISSION_PANEL = "submissionPanel";
    @Inject
	private IdeaService ideaService;
	@Inject
    private ApplicationPeriodService applicationPeriodService;

	public ProjectIdeaSubmissionPage(PageParameters pp) {
        super(pp);

        Long ideaId = getIdFromParameter(pp, PageParameterKeys.MAP.get(Idea.class));
        Long applicationPeriodId = getIdFromParameter(pp, PageParameterKeys.MAP.get(ApplicationPeriod.class));

        Idea idea;
        boolean isNewIdea;

        if (ideaId != null) {
            idea = ideaService.findOne(ideaId);
            isNewIdea = false;
            if (!idea.getCreator().equals(getUser()) || idea.getMatchStatus() != Idea.Status.UNMATCHED || idea.getApplicationPeriod().getEndDate().before(new Date())) {
                failAndRedirect();
            }
        } else {
            if (applicationPeriodId == null) {
                throw new RestartResponseException(ProjectIdeaStartPage.class);
            }
            final ApplicationPeriod applicationPeriod = applicationPeriodService.findOne(applicationPeriodId);
            if (applicationPeriod == null) {
                throw new RestartResponseException(ProjectIdeaStartPage.class);
            }
            idea = new Idea();
            isNewIdea = true;
            idea.setApplicationPeriod(applicationPeriod);
        }
        add(new ProjectIdeaSubmissionPanel(SUBMISSION_PANEL, new DetachableServiceModel<>(ideaService, idea), isNewIdea));
	}

    private Long getIdFromParameter(PageParameters pp, String parameter) {
        Long id = null;
        StringValue value = pp.get(parameter);
        if(!value.isNull()) {
            id = value.toLongObject();
        }
        return id;
    }

    private void failAndRedirect() {
		throw new RestartResponseException(getApplication().getApplicationSettings()
				.getAccessDeniedPage());
	}

}
