package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.applicationperiod.ApplicationPeriodInfoPanel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.List;

public class StudentApplicationPeriodInfoPanel extends Panel {
    @Inject
    private ApplicationPeriodService periodService;
    @Inject
    private IdeaService ideaService;

    static final String PERIOD_LIST = "currentPeriods";
    static final String EMPTY = "emptyLabel";
    static final String STUDENT_CONTAINER = "studentContainer";
    static final String SELECT_SUPERVISOR = "selectSupervisorIdeas";
    static final String CREATE = "createIdea";

    public StudentApplicationPeriodInfoPanel(String id) {
        super(id);

        IModel<List<StudentPeriod>> currentPeriods = new LoadableDetachableModel<List<StudentPeriod>>() {
            @Override
            protected List<StudentPeriod> load() {
                return periodService.getCurrentStudentPeriods(SciProSession.get().getUser());
            }
        };

        addListOfPeriods(currentPeriods);
    }

    private void addListOfPeriods(final IModel<List<StudentPeriod>> currentPeriods) {
        add(new AutoHidingListView<StudentPeriod>(PERIOD_LIST, currentPeriods) {

            @Override
            protected void populateItem(ListItem<StudentPeriod> item) {
                item.add(new ApplicationPeriodInfoPanel("info", new PropertyModel<>(item.getModel(), "applicationPeriod")));
                addStudentStuff(item);
            }

            private void addStudentStuff(final ListItem<StudentPeriod> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), item.getModelObject().getApplicationPeriod().getId());
                WebMarkupContainer studentContainer = new WebMarkupContainer(STUDENT_CONTAINER) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(!ideaService.authorParticipatingOnActiveIdea(SciProSession.get().getUser(), item.getModelObject().getApplicationPeriod()));
                    }
                };
                item.add(studentContainer);
                studentContainer.add(new BookmarkablePageLink<Void>(SELECT_SUPERVISOR, ProjectNewAllSupervisorIdeasPage.class, pp));
                studentContainer.add(new BookmarkablePageLink<Void>(CREATE, ProjectIdeaSubmissionPage.class, pp) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(item.getModelObject().fold(openPeriod -> true, exemptedPeriod -> true));
                    }
                });
            }
        });

        add(new Label(EMPTY, "No application period open") {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(currentPeriods.getObject().isEmpty());
            }
        });
    }
}
