package se.su.dsv.scipro.plagiarism;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.ExternalLink;
import org.apache.wicket.markup.html.link.Link;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.plagiarism.urkund.UrkundService;
import se.su.dsv.scipro.plagiarism.urkund.UrkundSubmission;
import se.su.dsv.scipro.session.SciProSession;

import javax.inject.Inject;

public class UrkundSubmissionPanel extends GenericPanel<FileDescription> {
    @Inject
    private PlagiarismControl plagiarismControl;
    @Inject
    private UrkundService urkundService;

    public UrkundSubmissionPanel(final String id, final IModel<FileDescription> model) {
        super(id, model);

        final AbstractReadOnlyModel<UrkundSubmission> submission = new AbstractReadOnlyModel<UrkundSubmission>() {
            @Override
            public UrkundSubmission getObject() {
                return urkundService.getSubmission(model.getObject()).orElseGet(() -> null);
            }

            @Override
            public void detach() {
                model.detach();
            }
        };
        add(new Details("submissionDetails", submission));

        final AbstractReadOnlyModel<Boolean> queued = new AbstractReadOnlyModel<Boolean>() {
            @Override
            public Boolean getObject() {
                return plagiarismControl.isQueued(model.getObject());
            }
        };
        final WebMarkupContainer notSubmitted = new WebMarkupContainer("notSubmitted", submission) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getDefaultModelObject() == null);
            }
        };
        final WebMarkupContainer notQueued = new WebMarkupContainer("notQueued") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!queued.getObject());
            }
        };
        notQueued.add(new Link<FileDescription>("submit", model) {
            @Override
            public void onClick() {
                plagiarismControl.submit(getModelObject(), SciProSession.get().getUser());
            }
        });
        notSubmitted.add(new Label("queued", new ResourceModel("queued", "The document is queued for text matching, check back later.")).add(new OppositeVisibility(notQueued)));
        notSubmitted.add(notQueued);
        add(notSubmitted);
    }

    private class Details extends WebMarkupContainer {
        private Details(final String id, final IModel<UrkundSubmission> submission) {
            super(id, submission);

            //add(new EnumLabel<UrkundSubmission.State>("state", new PropertyModel<>(submission, "state")));
            add(new Label("message", new PropertyModel<>(submission, "message")));
            add(new ExternalLink("report", new PropertyModel<>(submission, "reportUrl")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(getDefaultModelObject() != null);
                }
            });
            add(new Label("significance", new PropertyModel<>(submission, "significance")) {
                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(getDefaultModelObject() != null);
                }
            });
            add(new Link<FileDescription>("resubmit", new PropertyModel<>(submission, "fileDescription")) {
                @Override
                public void onClick() {
                    plagiarismControl.resubmit(getModelObject(), SciProSession.get().getUser());
                }

                @Override
                protected void onConfigure() {
                    super.onConfigure();
                    setVisibilityAllowed(submission.getObject().getState() == UrkundSubmission.State.SUBMISSION_FAILED);
                }
            });
        }

        @Override
        protected void onConfigure() {
            setVisibilityAllowed(getDefaultModelObject() != null);
        }
    }

    @Override
    protected void onConfigure() {
        super.onConfigure();
        setVisibilityAllowed(isVisibilityAllowed() && getModelObject() != null);
    }
}
