package se.su.dsv.scipro.profile;

import org.apache.wicket.injection.Injector;
import org.apache.wicket.request.resource.AbstractResource;
import org.apache.wicket.request.resource.IResource;
import org.apache.wicket.request.resource.ResourceReference;
import org.apache.wicket.util.time.Duration;
import se.su.dsv.scipro.daisyExternal.http.DaisyAPI;
import se.su.dsv.scipro.daisyExternal.http.PhotoResult;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.finalseminar.FinalSeminarService;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.project.ProjectTeamMemberRoles;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class DaisyPicture extends ResourceReference {
    public static final String USER_ID = "id";

    @Inject
    private ProjectService projectService;

    @Inject
    private FinalSeminarService finalSeminarService;

    @Inject
    private DaisyAPI daisyAPI;

    public DaisyPicture() {
        super("daisyPicture");
        Injector.get().inject(this);
    }

    @Override
    public IResource getResource() {
        return new AbstractResource() {
            @Override
            protected ResourceResponse newResourceResponse(final Attributes attributes) {
                final SciProSession session = SciProSession.get();
                if (!session.isLoggedIn()) {
                    return noPhoto();
                }

                final ProjectService.Filter filter = new ProjectService.Filter();
                filter.setSupervisor(session.getUser());
                filter.setRoles(Arrays.asList(ProjectTeamMemberRoles.CO_SUPERVISOR, ProjectTeamMemberRoles.REVIEWER));
                final List<Project> projects = projectService.findAll(filter);

                final Set<Long> projectRelations = projects
                        .stream()
                        .flatMap(p -> p.getMembers().stream())
                        .map(Member::getUser)
                        .map(User::getIdentifier)
                        .collect(Collectors.toSet());

                final Set<Long> seminarRelations = projects.stream()
                        .map(finalSeminarService::findByProject)
                        .filter(fs -> fs != null)
                        .flatMap(fs -> Stream.concat(fs.getOpponents().stream(), fs.getActiveParticipants().stream()))
                        .map(User::getIdentifier)
                        .collect(Collectors.toSet());

                final long userId = attributes.getParameters().get(USER_ID).toLong(0);
                final boolean alwaysShow = projectRelations.contains(userId) || seminarRelations.contains(userId);
                Long requesterDaisyId = session.getUser().getIdentifier();
                final PhotoResult photo = daisyAPI.getPhoto(userId, requesterDaisyId == null ? -1 : requesterDaisyId, alwaysShow);
                return photo.fold(this::noPhoto, this::noPhoto, this::showPhoto);
            }

            private ResourceResponse showPhoto(final InputStream inputStream) {
                final ResourceResponse resourceResponse = new ResourceResponse();
                resourceResponse.setWriteCallback(new WriteCallback() {
                    @Override
                    public void writeData(final Attributes attributes) throws IOException {
                        try { writeStream(attributes, inputStream); }
                        finally { inputStream.close(); }
                    }
                });
                resourceResponse.setCacheDuration(Duration.ONE_DAY);
                return resourceResponse;
            }

            private ResourceResponse noPhoto() {
                final ResourceResponse resourceResponse = new ResourceResponse();
                resourceResponse.setCacheDuration(Duration.NONE);
                resourceResponse.setWriteCallback(new WriteCallback() {
                    @Override
                    public void writeData(final Attributes attributes) throws IOException {
                        try (InputStream is = DaisyPicture.class.getResourceAsStream("anonymous.png")) {
                            writeStream(attributes, is);
                        }
                    }
                });
                return resourceResponse;
            }
        };
    }
}
