package se.su.dsv.scipro.project.pages;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.model.*;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.firstmeeting.FirstMeetingService;
import se.su.dsv.scipro.firstmeeting.ProjectFirstMeeting;

import javax.inject.Inject;
import java.util.Optional;

public class ProjectFirstMeetingPage extends AbstractProjectDetailsPage {
    @Inject
    private FirstMeetingService firstMeetingService;

    public ProjectFirstMeetingPage(final PageParameters pp) {
        super(pp);

        final IModel<Optional<ProjectFirstMeeting>> model = new LoadableDetachableModel<Optional<ProjectFirstMeeting>>() {
            @Override
            protected Optional<ProjectFirstMeeting> load() {
                return firstMeetingService.findByProject(projectModel.getObject());
            }
        };
        final IModel<ProjectFirstMeeting> model1 = new AbstractReadOnlyModel<ProjectFirstMeeting>() {
            @Override
            public ProjectFirstMeeting getObject() {
                return model.getObject().orElse(null);
            }
        };

        add(new WebMarkupContainer("firstMeeting", new CompoundPropertyModel<>(model1)) {
            {
                add(new DateLabel("date", DateStyle.DATE));
                add(new DateLabel("time", new PropertyModel<>(model1, "date"), DateStyle.TIME));
                add(new Label("room"));
                add(new Label("description"));
            }

            @Override
            protected void onConfigure() {
                setVisibilityAllowed(model.getObject().isPresent());
            }
        });


        add(new WebMarkupContainer("none") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(!model.getObject().isPresent());
            }
        });
    }
}
