package se.su.dsv.scipro.project.panels;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.model.util.SetModel;
import se.su.dsv.scipro.components.AddUsersPanel;
import se.su.dsv.scipro.components.InfoPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectService;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.HashSet;
import java.util.SortedSet;
import java.util.TreeSet;

public class SupervisorEditProjectPanel extends GenericPanel<Project> {

    @Inject
    private ProjectService projectService;
	public static final String TITLE = "title";
    public static final String CO_SUPERVISORS = "co_supervisors";
    public static final String FORM = "form";
    public static final String FEEDBACK = "feedback";

	private AddUsersPanel coSupervisors;

	public SupervisorEditProjectPanel(String id, IModel<Project> projectIModel) {
		super(id, projectIModel);
		Form form = new UpdateProjectForm(FORM, projectIModel);
		add(form);
        add(new InfoPanel("infoPanel", new ResourceModel("info.text")));
	}

	private class UpdateProjectForm extends Form<Project> {

		public UpdateProjectForm(String form, IModel<Project> projectIModel) {
			super(form, projectIModel);
			initComponents();
		}

		@Override
		public void onSubmit() {
			Project project = getModelObject();
			project.setCoSupervisors(updateProjectFollowers());
			project = projectService.save(project);
			setModelObject(project);
			info(getString("successfully_updated"));
		}

		private void initComponents() {
			add(new RequiredTextField<>(TITLE, new PropertyModel<>(getModel(), "title")));
			coSupervisors = new AddUsersPanel(CO_SUPERVISORS, new SetModel<>(new HashSet<>(getModel().getObject().getCoSupervisors())), AddUsersPanel.LookFor.EMPLOYEES);
			add(coSupervisors);

			FeedbackPanel feedbackPanel = new FeedbackPanel(FEEDBACK);
			add(feedbackPanel);
        }

        private SortedSet<User> updateProjectFollowers() {
			SortedSet<User> projectFollowers = new TreeSet<>(new User.ByNameComparator());
			projectFollowers.addAll(coSupervisors.getModelObject());
			return projectFollowers;
		}
	}
}
