package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.markup.repeater.data.IDataProvider;
import org.apache.wicket.model.IModel;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Iterator;

public class ProjectPartnerDataProvider implements IDataProvider<ProjectPartner> {

    private ProjectPartnerService projectPartnerService;
    private final IModel<ApplicationPeriod> applicationPeriod;
    private final IModel<ProjectType> model;
    private int days;

    @Override
    public void detach() {
        applicationPeriod.detach();
        model.detach();
    }

    public ProjectPartnerDataProvider(final ProjectPartnerService projectPartnerService, int days, final IModel<ApplicationPeriod> applicationPeriod, final IModel<ProjectType> model) {
        this.projectPartnerService = projectPartnerService;
        this.days = days;
        this.applicationPeriod = applicationPeriod;
        this.model = model;
    }

    @Override
    public Iterator<? extends ProjectPartner> iterator(long first, long count) {
        final PageRequest pageRequest = new PageRequest(first, count, new Sort(Sort.Direction.DESC, "dateCreated"));
        return projectPartnerService.getProjectPartnerInSpan(days, applicationPeriod.getObject(), model.getObject(), pageRequest).iterator();
    }

    @Override
    public long size() {
        return projectPartnerService.countProjectPartnerInSpan(days, applicationPeriod.getObject(), model.getObject());
    }

    @Override
    public IModel<ProjectPartner> model(ProjectPartner object) {
        return new DetachableServiceModel<>(projectPartnerService, object);
    }

}
