package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.components.AjaxDropDownChoice;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.match.AbstractAdminMatchPage;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.ApplicationPeriodService;
import se.su.dsv.scipro.security.auth.Authorization;
import se.su.dsv.scipro.security.auth.roles.Roles;

import javax.inject.Inject;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

@Authorization(authorizedRoles = {Roles.SYSADMIN})
public class SinglesPage extends AbstractAdminMatchPage {
    @Inject
    private ApplicationPeriodService applicationPeriodService;

    public SinglesPage() {
        final IModel<List<ApplicationPeriod>> allPeriods = getAllApplicationPeriods();
        final IModel<ApplicationPeriod> applicationPeriod = new DetachableServiceModel<>(applicationPeriodService);

        final WebMarkupContainer singles = new SinglesPanel("singles", applicationPeriod) {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getModelObject() != null);
            }
        };
        singles.setOutputMarkupPlaceholderTag(true);
        add(singles);

        add(new AjaxDropDownChoice<ApplicationPeriod>(
                "applicationPeriod",
                applicationPeriod,
                allPeriods,
                new ChoiceRenderer<>("name", "id")) {
            @Override
            public void onNewSelection(AjaxRequestTarget target, ApplicationPeriod objectSelected) {
                target.add(singles);
            }
        });
    }

    private IModel<List<ApplicationPeriod>> getAllApplicationPeriods() {
        return new LoadableDetachableModel<List<ApplicationPeriod>>() {
            @Override
            protected List<ApplicationPeriod> load() {
                List<ApplicationPeriod> list = applicationPeriodService.findAll();
                Collections.sort(list, Comparator.comparing(ApplicationPeriod::getStartDate).reversed());
                return list;
            }
        };
    }
}
