package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.MultiLineLabel;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.markup.html.panel.Fragment;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.BootstrapTabs;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.ListAdapterModel;
import se.su.dsv.scipro.data.FilteredListModel;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import java.util.List;
import java.util.Objects;

import static java.util.Comparator.comparing;
import static java.util.Comparator.reverseOrder;

public class SinglesPanel extends GenericPanel<ApplicationPeriod> {
    @Inject
    private ProjectPartnerService projectPartnerService;

    public SinglesPanel(final String id, final IModel<ApplicationPeriod> model) {
        super(id, model);

        final IModel<List<ProjectPartner>> singles = getSingles();

        add(new BootstrapTabs<ProjectType>("tabs", getMatchableTypes()) {
            @Override
            protected WebMarkupContainer getContent(final String id, final IModel<ProjectType> model) {
                return new Tab(id, getSinglesForType(singles, model));
            }

            @Override
            protected IModel<String> getLabelModel(final IModel<ProjectType> model) {
                return new PropertyModel<>(model, "name");
            }
        });
    }

    private class Tab extends Fragment {
        public Tab(final String id, final IModel<List<ProjectPartner>> singles) {
            super(id, "tab", SinglesPanel.this, singles);

            add(new PropertyListView<ProjectPartner>("singles", singles) {
                @Override
                protected void populateItem(final ListItem<ProjectPartner> item) {
                    item.add(new UserLinkPanel("user"));
                    item.add(new MultiLineLabel("infotext"));
                    item.add(new DateLabel("dateCreated", DateStyle.DATE));
                }
            });
        }
    }

    private IModel<List<ProjectPartner>> getSinglesForType(final IModel<List<ProjectPartner>> singles, final IModel<ProjectType> projectType) {
        return new FilteredListModel<ProjectPartner>(singles) {
            @Override
            protected boolean accept(final ProjectPartner projectPartner) {
                return Objects.equals(projectPartner.getProjectType(), projectType.getObject());
            }
        };
    }

    private IModel<List<ProjectPartner>> getSingles() {
        return new LoadableDetachableModel<List<ProjectPartner>>() {
            @Override
            protected List<ProjectPartner> load() {
                final List<ProjectPartner> partnersWithoutIdea = projectPartnerService.getPartnersWithoutIdea(getModelObject());
                partnersWithoutIdea.sort(comparing(ProjectPartner::getDateCreated, reverseOrder()));
                return partnersWithoutIdea;
            }
        };
    }

    private IModel<List<ProjectType>> getMatchableTypes() {
        final IModel<List<ProjectType>> matchableTypes = new ListAdapterModel<>(new PropertyModel<>(getModel(), "projectTypes"));
        return new FilteredListModel<ProjectType>(matchableTypes) {
            @Override
            protected boolean accept(final ProjectType projectType) {
                return projectType.getMaxAuthors() > 1;
            }
        };
    }
}
