package se.su.dsv.scipro.reviewer.timeline;

import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.*;

import javax.inject.Inject;
import java.util.*;
import java.util.stream.Collectors;

public class TimelinePanel extends Panel {
    @Inject
    private ReviewerInteractionService reviewerInteractionService;
    @Inject
    private FinalSeminarApprovalService finalSeminarApprovalService;
    @Inject
    private RoughDraftApprovalService roughDraftApprovalService;

    public TimelinePanel(final String id, final IModel<Project> model) {
        super(id, model);

        IModel<List<Event>> events = new LoadableDetachableModel<List<Event>>() {
            @Override
            protected List<Event> load() {
                Collection<Event> events = getForumEvents(model.getObject());

                Collection<Decision> finalSeminarDecisions = getFinalSeminarDecisions(model.getObject());

                Collection<Event> finalSeminarEvents = expandMultiple(finalSeminarDecisions);

                Collection<Decision> roughDraftDecisions = getRoughDraftDecisions(model.getObject());

                Collection<Event> roughDraftEvents = expandMultiple(roughDraftDecisions);

                List<Event> ev = new ArrayList<>();
                ev.addAll(events);
                ev.addAll(finalSeminarEvents);
                ev.addAll(roughDraftEvents);

                Collections.sort(ev, Collections.reverseOrder(Comparator.comparing(Event::when)));

                return ev;
            }
        };

        add(new ListView<Event>("events", events) {
            @Override
            protected void populateItem(final ListItem<Event> item) {
                item.add(item.getModelObject().component("event", item.getModel()));
            }
        });
    }

    private Collection<Event> getForumEvents(final Project project) {
        return reviewerInteractionService.getPosts(project).stream().map(Event.Message::new).collect(Collectors.toList());
    }

    private Collection<Decision> getFinalSeminarDecisions(final Project project) {
        return finalSeminarApprovalService.findBy(project)
                .map(FinalSeminarApproval::getHistory)
                .getOrElse(Collections::emptyList);
    }

    private Collection<Decision> getRoughDraftDecisions(final Project project) {
        return roughDraftApprovalService.findBy(project)
                .map(RoughDraftApproval::getHistory)
                .getOrElse(Collections::emptyList);
    }

    private Collection<Event> expandMultiple(final Collection<Decision> finalSeminarDecisions) {
        return finalSeminarDecisions.stream().flatMap(input -> expand(input).stream()).collect(Collectors.toList());
    }

    private List<Event> expand(final Decision decision) {
        switch (decision.getStatus()) {
            case APPROVED:
                return Arrays.<Event>asList(new Event.Requested(decision), new Event.Approved(decision));
            case REJECTED:
                return Arrays.<Event>asList(new Event.Requested(decision), new Event.Denied(decision));
            case UNDECIDED:
                return Collections.<Event>singletonList(new Event.Requested(decision));
            default:
                throw new Error();
        }
    }
}
