package se.su.dsv.scipro.settings.pages;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.head.CssHeaderItem;
import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.apache.wicket.request.resource.PackageResourceReference;
import org.apache.wicket.validation.validator.StringValidator;
import se.su.dsv.scipro.loginlogout.pages.SetPasswordPage;
import se.su.dsv.scipro.profile.UserProfileImage;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.settings.dataobjects.UserProfile;
import se.su.dsv.scipro.springdata.services.UserProfileService;
import se.su.dsv.scipro.system.Hash;
import se.su.dsv.scipro.system.HashService;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.AjaxConfirmationLink;

import javax.inject.Inject;

public class BasicProfilePage extends AbstractSettingsPage {
    static final String FEEDBACK = "feedback";
    static final String NAME = "name";
    static final String EMAIL = "email";
    static final String FORM = "form";
    static final String SKYPE = "skypeId";
    static final String PHONE = "phoneNumber";
    static final String OTHER = "otherInfo";
    static final int MAX_LENGTH = 200;

    @Inject
    private UserProfileService profileService;
    @Inject
    private HashService hashService;

    public BasicProfilePage() {
        add(new FeedbackPanel(FEEDBACK));

        IModel<UserProfile> profile = new LoadableDetachableModel<UserProfile>() {
            @Override
            protected UserProfile load() {
                return profileService.findByUser(getUser());
            }
        };
        add(new UserProfileImage("image", new LoadableDetachableModel<User>() {
            @Override
            public User load() {
                return getUser();
            }
        }, UserProfileImage.Size.LARGE));
        add(new ProfileForm(FORM, new CompoundPropertyModel<>(profile)));
    }

    private class ProfileForm extends Form<UserProfile> {

        public ProfileForm(String id, final IModel<UserProfile> model) {
            super(id, model);
            setMultiPart(true);
            add(new Label(NAME, getUser().getFirstName()));
            add(new Label(EMAIL, getUser().getEmailAddress()));
            add(new TextField<String>(SKYPE));
            add(new TextField<String>(PHONE));
            add(new TextArea<String>(OTHER).add(StringValidator.maximumLength(MAX_LENGTH)));

            add(new AjaxConfirmationLink<Void>("changePassword", "Are you sure?") {
                @Override
                public void onClick(AjaxRequestTarget target) {
                    final User user = SciProSession.get().getUser();
                    final Hash hash = hashService.createForUser(user);
                    final PageParameters pp = new PageParameters();
                    pp.add("hash", hash.getHash());
                    setResponsePage(SetPasswordPage.class, pp);
                }

                @Override
                protected void onConfigure() {
                    setVisibilityAllowed(SciProSession.get().getUser().getIdentifier()==null);
                }
            });
        }

        @Override
        protected void onSubmit() {
            profileService.save(getModelObject());
            info(getString("profile.saved"));
        }

    }

    @Override
    public void renderHead(final IHeaderResponse response) {
        super.renderHead(response);
        response.render(CssHeaderItem.forReference(new PackageResourceReference(BasicProfilePage.class, "imgareaselect-default.css")));
        response.render(JavaScriptHeaderItem.forReference(new PackageResourceReference(BasicProfilePage.class, "jquery.imgareaselect.pack.js")));
    }
}
