package se.su.dsv.scipro.match;

import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.User;

import java.util.Collections;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.match.SupervisorApplicationPeriodInfoPanel.PERIOD_LIST;
import static se.su.dsv.scipro.match.SupervisorApplicationPeriodInfoPanel.SELECT_STUDENT;

public class SupervisorApplicationPeriodInfoPanelTest extends SciProTest {
    @Test
    public void can_select_student_ideas_if_target_has_not_been_met() throws Exception {
        final ApplicationPeriod somePeriod = createSomePeriod();
        when(applicationPeriodService.getCurrentSupervisorPeriods(any(User.class))).thenReturn(Collections.singletonList(somePeriod));
        when(targetService.hasTargetsLeft(eq(somePeriod), any(User.class))).thenReturn(true);
        final SupervisorApplicationPeriodInfoPanel panel = tester.startComponentInPage(SupervisorApplicationPeriodInfoPanel.class);
        tester.assertVisible(path(panel, PERIOD_LIST, 0, SELECT_STUDENT));
    }

    @Test
    public void can_not_select_student_ideas_if_target_has_been_met() throws Exception {
        final ApplicationPeriod somePeriod = createSomePeriod();
        when(applicationPeriodService.getCurrentSupervisorPeriods(any(User.class))).thenReturn(Collections.singletonList(somePeriod));
        when(targetService.hasTargetsLeft(eq(somePeriod), any(User.class))).thenReturn(false);
        final SupervisorApplicationPeriodInfoPanel panel = tester.startComponentInPage(SupervisorApplicationPeriodInfoPanel.class);
        tester.assertInvisible(path(panel, PERIOD_LIST, 0, SELECT_STUDENT));
    }

    private ApplicationPeriod createSomePeriod() {
        final ApplicationPeriod somePeriod = new ApplicationPeriod();
        somePeriod.setId(1L);
        return somePeriod;
    }
}