package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.system.User;

import java.util.Arrays;

import static org.mockito.Mockito.when;

public class SinglesPanelTest extends SciProTest {
    private final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "bachelor");
    private final ProjectType master = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "master");
    private final ApplicationPeriod applicationPeriod = new ApplicationPeriod("Some period");

    @Before
    public void setUp() throws Exception {
        final User bill = User.builder().firstName("Bill").lastName("Gates").emailAddress("bill@example.com").build();
        final ProjectPartner billsAd = new ProjectPartner(bill);
        billsAd.setProjectType(bachelor);

        final User steve = User.builder().firstName("Steve").lastName("Jobs").emailAddress("steve@example.com").build();
        final ProjectPartner stevesPartner = new ProjectPartner(steve);
        stevesPartner.setProjectType(master);

        when(projectTypeService.findWithModule(ProjectModule.MATCH))
                .thenReturn(Arrays.asList(bachelor, master));
        when(projectPartnerService.getPartnersWithoutIdea(applicationPeriod))
                .thenReturn(Arrays.asList(billsAd, stevesPartner));
    }

    @Test
    public void renders() throws Exception {
        tester.startComponentInPage(new SinglesPanel("id", Model.of(applicationPeriod)));
    }
}